﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;


public class ProceduralChainV2 : MVRScript
{
    private Rigidbody RootObject = null;
    private ConfigurableJoint ChainLinkJoint = null;

    private Vector3 anchor;
    private Vector3 connectedanchor;
    private Transform ChainEnd = null;
    private int x = 0;
    private float MassScale = 100.0f;
    private float ConnectedMassScale = 90.0f;
    private float ChainLinkMass = 0.1f;
    private float AngularDamper = 1.0f;
    private float LinearDamper = 10.0f;
    private int ChainLinkCount = 10;
    private float ChainLinkScale = 1.0f;

    protected JSONStorableString jstring;
    protected UIDynamicButton UpdateChainLinksButton;
    protected UIDynamicButton UpdateChainParamsButton;
    protected UIDynamicSlider MassSlider;
    protected UIDynamicSlider MassScaleSlider;
    protected UIDynamicSlider ConnectedMassScaleSlider;
    protected UIDynamicSlider ChainLinkMassSlider;
    protected UIDynamicSlider AngularDamperSlider;
    protected UIDynamicSlider LinearDamperSlider;
    protected UIDynamicSlider ChainLinkCountSlider;
    protected UIDynamicSlider ChainLinkScaleSlider;
    protected UIDynamicTextField dtext;


    public override void Init()
    {
        try
        {
            JSONStorableFloat jfloat = new JSONStorableFloat("Mass Scale", 100f, MassScaleCallback, 0f, 5000f, true);
            RegisterFloat(jfloat);
            MassScaleSlider = CreateSlider(jfloat);

            jfloat = new JSONStorableFloat("Connected Mass Scale", 90f, ConnectedMassScaleCallback, 0f, 5000f, true);
            RegisterFloat(jfloat);
            MassScaleSlider = CreateSlider(jfloat);

            jfloat = new JSONStorableFloat("Chain Link Mass", 0.1f, ChainLinkMassCallback, 0f, 10f, true);
            RegisterFloat(jfloat);
            ChainLinkMassSlider = CreateSlider(jfloat);

            jfloat = new JSONStorableFloat("Angular Damper", 1.0f, AngularDamperCallback, 0f, 1000f, true);
            RegisterFloat(jfloat);
            AngularDamperSlider = CreateSlider(jfloat);

            jfloat = new JSONStorableFloat("Linear Damper", 10.0f, LinearDamperCallback, 0f, 1000f, true);
            RegisterFloat(jfloat);
            LinearDamperSlider = CreateSlider(jfloat);


            jfloat = new JSONStorableFloat("Chain Link Count", 10.0f, ChainLinkCountCallback, 0f, 100f, true);
            RegisterFloat(jfloat);
            ChainLinkCountSlider = CreateSlider(jfloat);

            jfloat = new JSONStorableFloat("Chain Link Scale", 1.0f, ChainLinkScaleCallback, 0.05f, 10f, true);
            RegisterFloat(jfloat);
            ChainLinkScaleSlider = CreateSlider(jfloat);

            UpdateChainLinksButton = CreateButton("Update Link Count");
            if (UpdateChainLinksButton != null)
            {
                UpdateChainLinksButton.button.onClick.AddListener(UpdateChainLinksButtonCallback);
                RegisterAction(new JSONStorableAction("UpdateChainLinks", UpdateChainLinksButtonCallback));
            }

            UpdateChainParamsButton = CreateButton("Update Parameter");
            if (UpdateChainParamsButton != null)
            {
                UpdateChainParamsButton.button.onClick.AddListener(UpdateChainParamsButtonCallback);
                RegisterAction(new JSONStorableAction("Update Parameter", UpdateChainParamsButtonCallback));

            }

            foreach (Rigidbody rb in containingAtom.rigidbodies)
            {
                SuperController.LogMessage(rb.name + " " + rb.position.ToString());
                if (rb.name == "object")
                    RootObject = rb;
            }

            if (RootObject == null)
            {
                SuperController.LogError("Missing RootObject! " + RootObject);
                return;
            }

            foreach (ConfigurableJoint cj in RootObject.GetComponentsInChildren<ConfigurableJoint>())
            {
                //SuperController.LogMessage("Joint: " + cj.name + " " + cj.transform.position.ToString() + " " + cj.GetInstanceID());
                if (cj.name == "ChainLink")
                {
                    ChainLinkJoint = cj;
                    break;
                }
            }

            if (ChainLinkJoint == null)
            {
                SuperController.LogMessage("No Chain link Joints found!" + ChainLinkJoint);
                return;
            }

            ChainLinkJoint.transform.localPosition = new Vector3(0.0f, -0.05f, 0.0f);
            anchor = ChainLinkJoint.anchor;
            connectedanchor = ChainLinkJoint.connectedAnchor;
            UpdateChainLinksButtonCallback();
        }

        catch (Exception e)
        {
            SuperController.LogError("Exception caught: " + e);
        }
    }


    void ReLinkChainLink(Transform toRelink, Transform newParent)
    {
        if (toRelink == null)
            return;

        if (newParent == null)
        {
            toRelink.parent = null;
            foreach (ConfigurableJoint cj in toRelink.GetComponents<ConfigurableJoint>())
                cj.connectedBody = null;

            return;
        }
        toRelink.SetParent(newParent, true);
        toRelink.localPosition = new Vector3(0.0f, -0.05f, 0.0f);
        foreach (ConfigurableJoint cj in toRelink.GetComponents<ConfigurableJoint>())
            cj.connectedBody = newParent.GetComponent<Rigidbody>();
    }

    Transform AddChainLink(Transform locparent)
    {
        Transform newtf = null;
        Rigidbody parentBody;
        x++;
        if (locparent == null)
            return null;
        parentBody = locparent.GetComponent<Rigidbody>();
        if (parentBody == null)
            return null;
        newtf = Transform.Instantiate(locparent, locparent);
        newtf.localRotation = Quaternion.Euler(0.0f, 90.0f, 0.0f);
        newtf.localPosition = new Vector3(0.0f, -0.05f, 0.0f);
        newtf.name = "ChainLink" + x;
        foreach (ConfigurableJoint cj in newtf.GetComponents<ConfigurableJoint>())
        {
            cj.connectedBody = parentBody;
        }
        return newtf;
    }

    void RemoveChainLinks(Transform toRemove)
    {
        for (int i = 0; i < toRemove.childCount; i++)
        {
            RemoveChainLinks(toRemove.GetChild(i));
        }
        SuperController.LogMessage("Destroing: " + toRemove.name + toRemove.GetComponent<Rigidbody>().name);
        DestroyImmediate(toRemove.gameObject);
        x--;
    }

    void Start()
    {

    }

    void Update()
    {
        if (ChainLinkJoint == null)
        {
            if (RootObject == null)
                return;

            foreach (ConfigurableJoint cj in RootObject.GetComponentsInChildren<ConfigurableJoint>())
            {
                //SuperController.LogMessage("Joint: " + cj.name + " " + cj.transform.position.ToString() + " " + cj.GetInstanceID());
                if (cj.name == "ChainLink")
                {
                    ChainLinkJoint = cj;
                    break;
                }
            }

            if (ChainLinkJoint == null)
            {
                SuperController.LogMessage("No Chain link Joints found!" + ChainLinkJoint);
                return;
            }

            ChainLinkJoint.transform.localPosition = new Vector3(0.0f, -0.05f, 0.0f);
            anchor = ChainLinkJoint.anchor;
            connectedanchor = ChainLinkJoint.connectedAnchor;
            UpdateChainLinksButtonCallback();
        }
    }

    void UpdateLinkParams(Transform toUpdate)
    {
        Rigidbody rb;
        ConfigurableJoint cj;

        for (int i = 0; i < toUpdate.childCount; i++)
        {
            UpdateLinkParams(toUpdate.GetChild(i));
        }

        rb = toUpdate.GetComponent<Rigidbody>();
        if (rb != null)
        {
            rb.mass = ChainLinkMass;
            rb.drag = LinearDamper;
            rb.angularDrag = AngularDamper;
        }

        cj = toUpdate.GetComponent<ConfigurableJoint>();
        if (cj != null)
        {
            cj.connectedMassScale = ConnectedMassScale;
            cj.massScale = MassScale;
        }

    }

    public void UpdateChainParamsButtonCallback()
    {

        UpdateLinkParams(ChainLinkJoint.transform);

    }

    public void UpdateChainLinksButtonCallback()
    {
        Transform newChainLink;
        Rigidbody rb;
        RootObject.transform.localScale = new Vector3(1.0f, 1.0f, 1.0f);
        if (ChainEnd != null)
            ReLinkChainLink(ChainEnd, null);
        if (ChainLinkJoint.transform.childCount > 0)
            RemoveChainLinks(ChainLinkJoint.transform.GetChild(0));
        newChainLink = ChainLinkJoint.transform;
        ChainLinkJoint.massScale = MassScale;
        ChainLinkJoint.connectedMassScale = ConnectedMassScale;
        rb = newChainLink.GetComponent<Rigidbody>();
        if (rb != null)
        {
            rb.mass = ChainLinkMass;
            rb.angularDrag = AngularDamper;
            rb.drag = LinearDamper;
        }
        for (int i = 0; i < ChainLinkCount; i++) // Verwende die neue ChainLinkCount-Variable
        {
            if (newChainLink != null)
                newChainLink = AddChainLink(newChainLink);
        }
        if (ChainEnd == null)
        {
            newChainLink.name = "ChainEnd";
            ChainEnd = newChainLink;
        }
        else
        {
            ReLinkChainLink(ChainEnd, newChainLink);
        }
        UpdateChainScale(new Vector3(ChainLinkScale, ChainLinkScale, ChainLinkScale));
    }

    private void UpdateChainScale(Vector3 newScale)
    {
        float newMass = 0.0f;

        if (newScale == null || RootObject == null || ChainLinkJoint == null)
            return;

        RootObject.transform.localScale = newScale;
        foreach (ConfigurableJoint cl in RootObject.GetComponentsInChildren<ConfigurableJoint>())
        {
            cl.anchor = anchor;
            cl.connectedAnchor = connectedanchor;
            newMass += cl.GetComponent<Rigidbody>().mass;
        }
        ChainLinkJoint.autoConfigureConnectedAnchor = false;
        ChainLinkJoint.transform.localPosition = new Vector3(0.0f, -0.05f * newScale.y, 0.0f);
        ChainLinkJoint.GetComponentInParent<Rigidbody>().mass = newMass * 10000;
        UpdateLinkParams(ChainLinkJoint.transform);
    }

    protected void ChainLinkScaleCallback(JSONStorableFloat jf)
    {
        try
        {
            if (jf != null)
            {
                UpdateChainScale(new Vector3(jf.val, jf.val, jf.val));
                ChainLinkScale = jf.val;
            }
        }
        catch (Exception e)
        {
            SuperController.LogError("Exception caught: " + e);
        }
    }

    protected void ChainLinkCountCallback(JSONStorableFloat jf)
    {
        ChainLinkCount = (int)jf.val;
    }

    protected void MassScaleCallback(JSONStorableFloat jf)
    {
        MassScale = jf.val;
    }

    protected void ConnectedMassScaleCallback(JSONStorableFloat jf)
    {
        ConnectedMassScale = jf.val;
    }


    protected void ChainLinkMassCallback(JSONStorableFloat jf)
    {
        ChainLinkMass = jf.val;
    }

    protected void AngularDamperCallback(JSONStorableFloat jf)
    {
        AngularDamper = jf.val;
    }

    protected void LinearDamperCallback(JSONStorableFloat jf)
    {
        LinearDamper = jf.val;
    }
}
